@@ -140,11 +140,6 @@ struct timeval startTime;
 bool		stop_now = false;
 #endif
 
-/*
- * If true, connection pool has been initialized.
- */
-static bool connection_pool_initialized = false;
-
 /*
 * child main loop
 */
@@ -228,7 +223,6 @@ do_child(int *fds)
 	{
 		child_exit(POOL_EXIT_AND_RESTART);
 	}
-	connection_pool_initialized = true;
 
 	/*
 	 * Open pool_passwd in child process.  This is necessary to avoid the file
@@ -407,6 +401,18 @@ do_child(int *fds)
 			backend_timer_expired = 0;
 		}
 
+		/*
+		 * Check whether failover/failback is ongoing and wait for it to
+		 * finish. If it actually happened, update the private backend status
+		 * because it is possible that a backend maybe went down.
+		 */
+		if (wait_for_failover_to_finish() < 0)
+			pool_initialize_private_backend_status();
+
+		/*
+		 * Connect to backend. Also do authentication between
+		 * frontend <--> pgpool and pgpool <--> backend.
+		 */
 		backend = get_backend_connection(child_frontend);
 		if (!backend)
 		{
@@ -766,11 +772,8 @@ read_startup_packet(POOL_CONNECTION * cp)
 	}
 
 	/* The database defaults to their user name. */
-	if (sp->database == NULL)
-		sp->database = pstrdup(sp->user);
-	else if (sp->database[0] == '\0')
+	if (sp->database == NULL || sp->database[0] == '\0')
 	{
-		pfree(sp->database);
 		sp->database = pstrdup(sp->user);
 	}
 
@@ -835,7 +838,49 @@ connect_using_existing_connection(POOL_CONNECTION * frontend,
 
 	if (MAJOR(backend) == 3)
 	{
-		/* Send parameter status message to frontend. */
+		char		command_buf[1024];
+
+		/*
+		 * If we have received application_name in the start up packet, we
+		 * send SET command to backend. Also we add or replace existing
+		 * application_name data.
+		 */
+		if (sp->application_name)
+		{
+			snprintf(command_buf, sizeof(command_buf), "SET application_name TO '%s'", sp->application_name);
+
+			for (i = 0; i < NUM_BACKENDS; i++)
+			{
+				if (VALID_BACKEND(i))
+				{
+					/*
+					 * We want to catch and ignore errors in do_command if a
+					 * backend is just going down right now. Otherwise
+					 * do_command raises an error and disconnects the
+					 * connection to frontend. We can safely ignore error from
+					 * "SET application_name" command if the backend goes
+					 * down.
+					 */
+					PG_TRY();
+					{
+						do_command(frontend, CONNECTION(backend, i),
+								   command_buf, MAJOR(backend),
+								   MAIN_CONNECTION(backend)->pid,
+								   MAIN_CONNECTION(backend)->key, 0);
+					}
+					PG_CATCH();
+					{
+						/* ignore the error message */
+						MemoryContextSwitchTo(oldContext);
+						FlushErrorState();
+					}
+					PG_END_TRY();
+				}
+			}
+			pool_add_param(&MAIN(backend)->params, "application_name", sp->application_name);
+			set_application_name_with_string(sp->application_name);
+		}
+
 		send_params(frontend, backend);
 	}
 
@@ -963,6 +1008,10 @@ found:
 	}
 }
 
+/*
+ * Copy startup packet and return it.
+ * palloc is used.
+ */
 static StartupPacket *
 StartupPacketCopy(StartupPacket *sp)
 {
@@ -992,6 +1041,10 @@ StartupPacketCopy(StartupPacket *sp)
 	return new_sp;
 }
 
+/*
+ * Create a new connection to backend.
+ * Authentication is performed if requested by backend.
+ */
 static POOL_CONNECTION_POOL * connect_backend(StartupPacket *sp, POOL_CONNECTION * frontend)
 {
 	POOL_CONNECTION_POOL *backend;
@@ -1050,6 +1103,7 @@ static POOL_CONNECTION_POOL * connect_backend(StartupPacket *sp, POOL_CONNECTION
 																ALLOCSET_DEFAULT_SIZES);
 		oldContext = MemoryContextSwitchTo(frontend_auth_cxt);
 
+		/* do authentication against backend */
 		pool_do_auth(frontend, backend);
 
 		MemoryContextSwitchTo(oldContext);
@@ -1207,6 +1261,9 @@ static RETSIGTYPE authentication_timeout(int sig)
 	child_exit(POOL_EXIT_AND_RESTART);
 }
 
+/*
+ * Enable authentication timeout.
+ */
 static void
 enable_authentication_timeout(void)
 {
@@ -1216,6 +1273,9 @@ enable_authentication_timeout(void)
 	alarm_enabled = true;
 }
 
+/*
+ * Disable authentication timeout.
+ */
 static void
 disable_authentication_timeout(void)
 {
@@ -1226,7 +1286,9 @@ disable_authentication_timeout(void)
 	}
 }
 
-
+/*
+ * Send parameter status message to frontend.
+ */
 static void
 send_params(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend)
 {
@@ -1289,9 +1351,13 @@ child_will_go_down(int code, Datum arg)
 	}
 
 	/* let backend know now we are exiting */
-	if (connection_pool_initialized)
+	if (pool_connection_pool)
 		close_all_backend_connections();
 }
+
+/*
+ * Exit child process. Can be called from signal handlers.
+ */
 void
 child_exit(int code)
 {
@@ -1461,7 +1527,6 @@ check_restart_request(void)
  * to connect, on successful connection returns the socket descriptor
  * and returns -1 if timeout has occurred
  */
-
 static int
 wait_for_new_connections(int *fds, SockAddr *saddr)
 {
@@ -1640,6 +1705,11 @@ retry_accept:
 		pause();
 	}
 
+	/* wait for failover/failback to finish */
+	if (wait_for_failover_to_finish() < 0)
+		/* failover/failback occurred. Update private backend status */
+		pool_initialize_private_backend_status();
+
 	afd = accept(fd, (struct sockaddr *) &saddr->addr, &saddr->salen);
 
 	save_errno = errno;
@@ -1729,10 +1799,9 @@ check_config_reload(void)
 		if (pool_config->enable_pool_hba)
 		{
 			load_hba(get_hba_file_name());
+			if (strcmp("", pool_config->pool_passwd))
+				pool_reopen_passwd_file();
 		}
-		if (strcmp("", pool_config->pool_passwd))
-			pool_reopen_passwd_file();
-
 		got_sighup = 0;
 	}
 }
@@ -1888,6 +1957,10 @@ get_connection(int front_end_fd, SockAddr *saddr)
 	return cp;
 }
 
+/*
+ * Connect to backend. Also do authentication between client <--> pgpool and
+ * pgpool <--> backend.
+ */
 static POOL_CONNECTION_POOL *
 get_backend_connection(POOL_CONNECTION * frontend)
 {
@@ -1949,6 +2022,9 @@ retry_startup:
 										ALLOCSET_DEFAULT_SIZES);
 		MemoryContext oldContext = MemoryContextSwitchTo(frontend_auth_cxt);
 
+		/*
+		 * Do frontend <-> pgpool authentication based on pool_hba.conf.
+		 */
 		ClientAuthentication(frontend);
 
 		MemoryContextSwitchTo(oldContext);
@@ -2025,7 +2101,10 @@ retry_startup:
 
 	if (backend == NULL)
 	{
-		/* create a new connection to backend */
+		/*
+		 * Create a new connection to backend.
+		 * Authentication is performed if requested by backend.
+		 */
 		backend = connect_backend(sp, frontend);
 	}
 	else
